"""
Plain interface to names in the mapnik gis db.
"""

import Tkinter
from tkSimpleDialog import Dialog

import mapnik
import psycopg2

from pysmap import settings


gisDSN = settings.get("db", "dsn")


class GisContext(object):
	"""is a container for global info related to mapnik and the GIS db.
	"""
	def __init__(self, gisDSN, projection):
		self.connection = psycopg2.connect(gisDSN)
		self.proj = projection


class Settlements(object):
	"""is a wrapper for the results of a query for villages, etc..
	"""
	def __init__(self, gisContext, name):
		self.gisContext = gisContext
		cursor = gisContext.connection.cursor()
		cursor.execute("SELECT x(p), y(p), place, name FROM ("
			"SELECT way as p, place, name FROM planet_osm_point"
				" WHERE name LIKE %(name)s  AND place IS NOT NULL"
				" LIMIT 21) AS q", {"name": name})
		self._process(cursor.fetchall())

	def __iter__(self):
		return iter(self.data)

	def __len__(self):
		return len(self.data)

	def __getitem__(self, index):
		return self.data[index]

	def _process(self, queryResult):
		self.data = []
		for x, y, type, name in queryResult:
			pt = self.gisContext.proj.inverse(mapnik.Coord(x, y))
			self.data.append(("%s (%s)"%(name, type), pt.x, pt.y))


class GeoBrowser(Dialog):
	def __init__(self, parent, geoResults):
		self.geoResults = geoResults
		self.placeName, self.geoCoos = None, None
		Dialog.__init__(self, parent, "Choose Location")
	
	def body(self, master):
		scrollbar = Tkinter.Scrollbar(master, orient=Tkinter.VERTICAL)
		self.geoBrowser = Tkinter.Listbox(master, font=("Helvetica", 10),
			exportselection=0, selectmode=Tkinter.SINGLE,
			yscrollcommand=scrollbar.set, width=40, height=10)
		for res in self.geoResults:
			self.geoBrowser.insert(Tkinter.END, res[0])
		scrollbar.config(command=self.geoBrowser.yview)
		self.geoBrowser.grid(row=0, column=0, sticky=Tkinter.NW+Tkinter.SE)
		scrollbar.grid(row=0, column=1, sticky=Tkinter.N+Tkinter.S)
		self.rowconfigure(0, weight=1)
		self.columnconfigure(0, weight=1)
	
	def apply(self):
		sel = self.geoBrowser.curselection()
		if sel!=():
			item = self.geoResults[int(sel[0])]
			self.placeName = item[0]
			self.geoCoos = item[1:]


def _test():
	import mapview
	root = Tkinter.Tk()
	context = GisContext(gisDSN, mapview.mapnikProjection)
	sq = Settlements(context, "%heim")
	dia = GeoBrowser(root, sq)
	root.mainloop()
	print dia.placeName, dia.geoCoos


if __name__=="__main__":
	_test()
